
/*
 * $Id: ErrorStrTbl.cc,v 1.4.6.1 1999/11/20 18:10:52 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include <errno.h>
#include <string.h>

#include "ILog.h"
#include "OLog.h"
#include "ErrorStrTbl.h"

ErrorStrTbl::ErrorStrTbl(): theErrnoErrStr("unknown error") {
	fillUpTo(255);
}

ErrorStrTbl::~ErrorStrTbl() {
	reset();
}

void ErrorStrTbl::reset() {
	while (count()) delete pop();
}

int ErrorStrTbl::maxErrNo() const {
	return theCount-1;
}

const String &ErrorStrTbl::operator [](int errNo) const {
	if (errNo >= 0 && errNo < theCount) {
		Assert(item(errNo));
		return *item(errNo);
	}
	return theErrnoErrStr;
}

String &ErrorStrTbl::str(int errNo) {
	fillUpTo(errNo);
	Assert(item(errNo));
	return *item(errNo);
}

void ErrorStrTbl::fillUpTo(int lastIdx) {
	stretch(lastIdx+1);
	for (int e = theCount; e <= lastIdx; ++e) {
		Assert(e == theCount);
		errno = 0;
		const char *s = strerror(e);
		//cerr << here << e << ": " << s << " - " << error(errno) << endl;
		append(new String(s && !errno ? s : (const char*)theErrnoErrStr));
	}
}

ErrorStrTbl &ErrorStrTbl::operator =(const ErrorStrTbl &est) {
	if (this != &est) {
		reset();
		for (int e = 0; e <= est.maxErrNo(); ++e)
			append(new String(est[e]));
	}
	return *this;
}

OLog &ErrorStrTbl::store(OLog &ol) const {
	ol << theCount;
	for (int e = 0; e < theCount; ++e)
		ol << *item(e);
	ol << theErrnoErrStr;
	return ol;
}

ILog &ErrorStrTbl::load(ILog &il) {
	int cnt;
	il >> cnt;
	if (cnt < 0) {
		cerr << here << "corrupted log entry? (errno strings count: " << cnt << ")" << endl;
		exit(-2);
	}

	reset();
	stretch(cnt);
	for (int e = 0; e < cnt; ++e) {
		String *s = new String;
		il >> *s;
		append(s);
	}
	Assert(theCount == cnt);

	il >> theErrnoErrStr;

	return il;
}
