
/*
 * $Id: HRStat.cc,v 1.3.6.3 1999/12/20 15:32:59 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "ILog.h"
#include "OLog.h"
#include "HRStat.h"
#include "gadgets.h"


HRStat::HRStat() {
	reset();
}

void HRStat::reset() {
	theHits.reset();
	theMisses.reset();
}

CntSzStat HRStat::xacts() const { 
	CntSzStat res = theHits;
	res += theMisses;
	return res;
}

OLog &HRStat::store(OLog &log) const {
	return log << theHits << theMisses;
}

ILog &HRStat::load(ILog &log) {
	return log >> theHits >> theMisses;
}

double HRStat::dhr() const {
	return Ratio(hits().count(), xacts().count());
}

double HRStat::bhr() const {
	const BigSize denom = theHits.size() + theMisses.size();
	return denom != 0 ? theHits.size() / denom : -1;
}

double HRStat::dhp() const {
	return Percent(hits().count(), xacts().count());
}

double HRStat::bhp() const {
	const BigSize denom = theHits.size() + theMisses.size();
	return denom != 0 ? 100. * (theHits.size() / denom) : -1;
}

HRStat &HRStat::operator +=(const HRStat &hr) {
	theHits += hr.theHits;
	theMisses += hr.theMisses;
	return *this;
}

ostream &HRStat::print(ostream &os, const String &hit, const String &miss, const String &pfx = "") const {
	os << pfx << hit << ".ratio.obj: \t " << dhp() << endl;
	os << pfx << hit << ".ratio.byte:\t " << bhp() << endl;
	theHits.print(os, pfx + hit + ".");
	theMisses.print(os, pfx + miss + ".");
	return os;
}
