
/*
 * $Id: EmbedContMdl.h,v 1.2.2.2 1999/10/25 23:25:15 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_EMBEDDED_CONTENT_MODEL_H
#define PG_EMBEDDED_CONTENT_MODEL_H

#include "Array.h"

class WrBuf;
class RndGen;
class RndDistr;
class ContBuildIter;
class ContentCfg;
class ContentSym;


// simulates embedded content 
// (simulates various *ML tags that allow to "embed" content, to be precise)

class EmbedContMdl {
	public:
		EmbedContMdl();

		void configure(const ContentSym *cfg);

		// called by Server through ContentCfg
		double compContPerCall(const ContentCfg *cc) const;
		void noteNewContProb(ContentCfg *cc, double newProb);

		// called by xaction that builds content
		ContBuildIter *getIterator(Size contSize, WrBuf *buf);
		void putIterator(ContBuildIter *i);

		// called by content build iterator
		int embedGoal(RndGen &rng);
		int embedOid(RndGen &rng, int oid, int &embedType, int &newOidCount);

	protected:
		void configureEmbedCfgs(const ContentSym *cfg);
		bool find(int cfgId, int &idx) const;

	protected:
		Array<const ContentCfg*> theCfgs; // embedded objs configs
		Array<double> theProbs;       // probability of selecting a cfg
		Array<double> theNewContProb; // prob. of producing new content when producing an embedded oid
		RndDistr *theCfgSel;          // selector for cfgs
		RndDistr *theEmbedCount;      // objs per container
		
		double theEmbedCountMean;     // cached mean of theEmbedCount
};

#endif
