
/*
 * $Id: GoalRec.h,v 1.3.6.2 1999/12/19 06:52:34 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_GOAL_RECORD_CLASS_H
#define PG_GOAL_RECORD_CLASS_H

#include "LogObj.h"
#include "BigSize.h"

class String;

// used by GoalRec
class ErrGoalRec: public LogObj {
	public:
		ErrGoalRec();

		operator void*() const { return (theCount >= 0 || theRatio >= 0) ? (void*)-1 : 0; }
		int count() const { return theCount; }
		double ratio() const { return theRatio; }

		ErrGoalRec &operator +=(const ErrGoalRec &e);

		virtual OLog &store(OLog &) const;
		virtual ILog &load(ILog &);

		ostream &print(ostream &os) const;
		ostream &print(ostream &os, const String &pfx) const;

	public: /* for logging/config only */
		double theRatio;
		int theCount;
};

// log image of Goal
class GoalRec: public LogObj {
	public:
		GoalRec();

		operator void *() const; // true if positive goal is set
		Time duration() const { return theDuration; }
		int xactCnt() const { return theXactCnt; }
		const BigSize &fillSz() const { return theFillSz; }
		const ErrGoalRec &errs() const { return theErrs; }

		void concat(const GoalRec &g);
		void merge(const GoalRec &g);

		virtual OLog &store(OLog &) const;
		virtual ILog &load(ILog &);

		ostream &print(ostream &os) const;
		ostream &print(ostream &os, const String &pfx) const;

	protected:
		void join(const GoalRec &g);

	protected:
		Time theDuration;
		int theXactCnt;
		BigSize theFillSz;
		ErrGoalRec theErrs;
};

inline ostream &operator <<(ostream &os, const GoalRec &gr) { return gr.print(os); }

#endif
