
/*
 * $Id: Histogram.h,v 1.8.6.1 1999/12/19 06:22:29 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef HISTOGRAM_CLASS_H
#define HISTOGRAM_CLASS_H

#include "Array.h"
#include "AggrStat.h"
#include "ILog.h"
#include "OLog.h"
#include "LogObj.h"

class HistogramConstIter;

// general purpose histogram abstraction
// value range: [min, max)
class Histogram: public LogObj {
	friend HistogramConstIter;

	public:
		typedef int Val; // change to double if needed

	public:
		Histogram();
		Histogram(Val aMin, Val aMax, int binCount);
		virtual ~Histogram() {}

		void limits(Val aMin, Val aMax, int binCount);
		virtual void reset();

		void record(Val v);

		virtual OLog &store(OLog &log) const;
		virtual ILog &load(ILog &);

		virtual void add(const Histogram &h);

		const AggrStat &stats() const { return theStats; }

		// extreme bins' counters
		bool extreme(int bin) const { return !bin || bin == theBinMax; }
		int underCount() const { return theBins[0]; }
		int overCount() const { return theBins[theBinMax]; }

		ostream &print(ostream &os, const String &pfx = "") const;
		void report(double step, ostream &os) const;

	protected:
		virtual const char *type() const = 0; // val2bin type
		virtual int val2Bin(Val v) const = 0; // value is already offset
		virtual Val bin2Val(int b) const = 0; // does not offset value

		Val extract(int b) const { return bin2Val(b-1) + theValMin; }
		int count(int b) const { return theBins[b]; }

	private:
		Histogram(const Histogram &) {}

	protected:
		Array<int> theBins; // counters are stored here
		AggrStat theStats;
		Val theValMin;
		Val theValMax;
		int theBinMax;
};


// a virtual representation of a histogram "bin"
class HistogramBin {
	public:
		HistogramBin() { reset(); }
		void reset() { idx = count = accCount = 0; min = sup = -1; }

		HistogramBin &operator +=(const HistogramBin &b);

	public:
		Histogram::Val min;
		Histogram::Val sup; // max+1
		int idx;
		int count;
		int accCount; // cumulative count
};

// constant iterator for a histogram
class HistogramConstIter {
	public:
		HistogramConstIter(const Histogram &aHist);

		operator void*() const { return theBin.idx <= theHist.theBinMax ? (void*)-1 : 0; }
		const HistogramBin &operator *() const { return theBin; }
		HistogramConstIter &operator ++();

	private:
		void sync();

	protected:
		const Histogram &theHist;
		HistogramBin theBin; // current bin;
};


// builds percentiles array with a given step (1% default)
extern void Percentiles(const Histogram &, Array<HistogramBin> &percs, double step = 0.01);



#endif
