
/*
 * $Id: UniqId.h,v 1.3 1999/09/26 23:36:23 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_UNIQ_ID_CLASS_H
#define PG_UNIQ_ID_CLASS_H

class ostream;
class String;
class OLog;
class ILog;


// we often need an id with these characteristics:
//     - truly unique across hosts, processes, time (within ttl), etc
//     - at least one year ttl
//     - compact to store
//     - fast to produce [possibly in huge quantaties!]
//     - fast to compare on equality
//     - has one and only one "permutated" value that
//       cannot be generated by means other than "mutating" an id

class UniqId {
	public:
		static UniqId Create() { return UniqId().create(); }

	public:
		UniqId();
		UniqId(int aSecs, int aMix, int aCnt);

		void clear() { theCnt = 0; }
		UniqId &create(); // returns itself
		operator void*() const { return theCnt > 0 ? (void*)-1 : 0; } // created?

		bool mutant() const { return theCnt & 1; } // is a mutatant?
		inline bool myMutant(const UniqId &mid) const;
		inline UniqId genMutant() const; // produce a mutant

		// returns a not so unique integer
		int toInt() const;

		inline bool operator ==(const UniqId &id) const;
		inline bool operator !=(const UniqId &id) const;

		ostream &print(ostream &os) const;
		OLog &store(OLog &ol) const;
		ILog &load(ILog &il);

	protected:
		int theSecs;    // seconds since Nov 29 06:21:18 MST 1998
		int theMix;     // usec and process id
		int theCnt;     // instance
};


// handy to get string representation; not optmized
extern String Id2Str(const UniqId &id);


/* inline definitions */

inline
ostream &operator <<(ostream &os, const UniqId &id) { return id.print(os); }

inline
OLog &operator <<(OLog &ol, const UniqId &id) { return id.store(ol); }

inline
ILog &operator >>(ILog &il, UniqId &id) { return id.load(il); }

inline
UniqId UniqId::genMutant() const { 
	return UniqId(theSecs, theMix, ~theCnt & 0x7fffffff);
}

inline
bool UniqId::myMutant(const UniqId &m) const {
	return (m.theCnt == (~theCnt & 0x7fffffff)) &&
		theMix == m.theMix &&
		theSecs == m.theSecs;
}

inline
bool UniqId::operator ==(const UniqId &id) const {
	return theCnt == id.theCnt && // most selective first
		theMix == id.theMix &&
		theSecs == id.theSecs;
}

inline
bool UniqId::operator !=(const UniqId &id) const {
	return !(*this == id);
}


#endif
