
/*
 * $Id: PglBoolSym.cc,v 1.4 1999/09/07 17:47:41 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "pgl.h"

#include "PglBoolSym.h"


BoolSym::BoolSym(bool aVal = -1): ExpressionSym("bool"), theVal(aVal) {
}

bool BoolSym::isA(const String &type) const {
	return ExpressionSym::isA(type) ||
		type == "Bool" || type == "bool";
}

SynSym *BoolSym::clone(const String &type) const {
	if (isA(type))
		return new BoolSym(theVal);
	return ExpressionSym::clone(type);
}

ExpressionSym *BoolSym::unOper(const String &op) const {
	if (op == "!")
		return new BoolSym(!theVal);
	else
		return ExpressionSym::unOper(op);
}


ExpressionSym *BoolSym::bnOper(const String &op, const SynSym &s) const {
	if (op.len() < 1 || op.len() > 2 || !(op == "^" || op[0] == op[1]))
		return ExpressionSym::bnOper(op, s);

	bool otherVal = false;

	if (s.isA("bool")) {
		otherVal = ((const BoolSym&)s.cast("bool")).val();
	} else 
	if (BoolSym *is = (BoolSym*)s.clone("bool")) {
		otherVal = is->val();
		delete is;
	} else
		return ExpressionSym::bnOper(op, s);

	switch (op[0]) {
		case '|': return new BoolSym(theVal || otherVal);
		case '&': return new BoolSym(theVal && otherVal);
		case '^': return new BoolSym(theVal ^ otherVal);
		default: return ExpressionSym::bnOper(op, s);
	}
}

ostream &BoolSym::print(ostream &os, const String &) const {
	return os << (theVal ? "true" : "false");
}
