
/*
 * $Id: PglNumSym.cc,v 1.5 1999/09/07 17:47:45 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "pgl.h"

#include <math.h>

#include "PglNumSym.h"


NumSym::NumSym(double aVal): ExpressionSym("float"), theVal(aVal) {
}

bool NumSym::isA(const String &type) const {
	return ExpressionSym::isA(type) ||
		type == "Num" || type == "float";
}


SynSym *NumSym::clone(const String &type) const {
	if (isA(type))
		return new NumSym(theVal);

	return ExpressionSym::clone(type);
}

ExpressionSym *NumSym::unOper(const String &op) const {
	if (*op == '+')
		return new NumSym(+theVal);
	else
	if (*op == '-')
		return new NumSym(-theVal);
	else
		return ExpressionSym::unOper(op);
}


ExpressionSym *NumSym::bnOper(const String &op, const SynSym &s) const {
	if (strlen(op) != 1)
		return ExpressionSym::bnOper(op, s);

	double otherVal = 0;
	
	if (s.isA("float")) {
		otherVal = ((const NumSym&)s.cast("float")).theVal;
	} else {
		NumSym *fs = (NumSym*)s.clone("float");
		if (!fs)
			return ExpressionSym::bnOper(op, s);
		otherVal = fs->theVal;
		delete fs;
	}

	if (*op == '/' && fabs(otherVal) < 1e-8)
		cerr << theLoc << "denominator is "
			<< (otherVal ? "[virtually] " : "")
			<< "zero" << endl << xexit;

	switch (*op) {
		case '+': return new NumSym(theVal + otherVal);
		case '-': return new NumSym(theVal - otherVal);
		case '*': return new NumSym(theVal * otherVal);
		case '/': return new NumSym(theVal / otherVal);
		case '^': return new NumSym(pow(theVal, otherVal));
		default: return ExpressionSym::bnOper(op, s);
	}
}

ostream &NumSym::print(ostream &os, const String &) const {
	return os << theVal;
}
