
/*
 * $Id: RobotSym.cc,v 1.8.2.7 1999/12/27 07:36:18 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "pgl.h"

#include "String.h"
#include "TblDistr.h"
#include "PglBoolSym.h"
#include "PglIntSym.h"
#include "PglNumSym.h"
#include "PglRateSym.h"
#include "PglRec.h"
#include "PglPopModelSym.h"
#include "PglStringSym.h"
#include "PglArraySym.h"
#include "PglNetAddrSym.h"
#include "RobotSym.h"
#include "rndDistrs.h"


RobotSym::RobotSym(): AgentSym("Robot") {
	// add fields

	theRec->bAdd("addr[]", "origins", 0);
	theRec->bAdd("time", "launch_win", 0);
	theRec->bAdd("int", "cache_cap", 0);
	theRec->bAdd("rate", "req_rate", 0);
	theRec->bAdd("time_distr", "req_inter_arrival", 0);
	theRec->bAdd("float", "public_interest", 0);
	theRec->bAdd("float", "recurrence", 0);
	theRec->bAdd("float", "embed_recur", 0);
	theRec->bAdd("PopModel", "pop_model", 0);
	theRec->bAdd("bool", "unique_urls", 0);
	theRec->bAdd("int", "open_conn_lmt", 0);
	theRec->bAdd("int", "wait_xact_lmt", 0);
}

RobotSym::RobotSym(const String &aType, PglRec *aRec): AgentSym(aType, aRec) {
}

bool RobotSym::isA(const String &type) const {
	return AgentSym::isA(type) || type == "Robot";
}

SynSym *RobotSym::clone(const String &type) const {
	if (isA(type))
		return new RobotSym(this->type(), theRec->clone());
	return AgentSym::clone(type);
}

bool RobotSym::reqRate(double &rate) const {
	SynSymTblItem *rri = 0;
	Assert(theRec->find("req_rate", rri));
	if (rri->sym())
		rate = ((const RateSym&)rri->sym()->cast("rate")).val();
	return rri->sym();
}

RndDistr *RobotSym::reqInterArrival() const {
	RndDistr *iad = getDistr("req_inter_arrival");
	double rr = -1;
	if (reqRate(rr) && iad)
		cerr << loc() << ": cannot specify both request rate"
			<< " and inter-arrival distribution" << endl << xexit;

	if (iad)
		return iad;

	if (rr <= 0)
		cerr << loc() << ": request rate for robot " << kind() << " is not positive" << endl << xexit;

	// poisson request stream (default) is modeled using an exponential
	// distribution with a mean of a given request rate
	static RndGen rng; // all clients should share one rng
	return new ExpDistr(&rng, 1/rr);
}

Time RobotSym::launchWin() const {
	return getTime("launch_win");
}

bool RobotSym::cacheCap(int &cap) const {
	return getInt("cache_cap", cap);
}

bool RobotSym::publicRatio(double &ratio) const {
	return getDouble("public_interest", ratio);
}

bool RobotSym::recurRatio(double &ratio) const {
	return getDouble("recurrence", ratio);
}

bool RobotSym::embedRecurRatio(double &ratio) const {
	return getDouble("embed_recur", ratio);
}

PopModel *RobotSym::popModel() const {
	SynSymTblItem *pmi = 0;
	Assert(theRec->find("pop_model", pmi));
	return pmi->sym() ?
		((const PopModelSym&)pmi->sym()->cast("PopModel")).val() : 0;
}

bool RobotSym::uniqueUrls(bool &set) const {
	return getBool("unique_urls", set);
}

bool RobotSym::openConnLimit(int &lmt) const {
	return getInt("open_conn_lmt", lmt);
}

bool RobotSym::waitXactLimit(int &lmt) const {
	return getInt("wait_xact_lmt", lmt);
}

bool RobotSym::origins(Array<NetAddr*> &addrs, RndDistr *&selector) const {
	SynSymTblItem *oi = 0;
	Assert(theRec->find("origins", oi));
	if (!oi->sym())
		return false; // undefined

	// build address array and selector
	ArraySym &os = (ArraySym&)oi->sym()->cast("Array");
	selector = os.makeSelector(kind() + "-origins");
	for (int i = 0; i < os.count(); ++i) {
		NetAddr addr = ((const NetAddrSym&)os[i]->cast("addr")).val();
		addrs.append(new NetAddr(addr));
	}

	return true;
}
