
/*
 * $Id: Poll.cc,v 1.5.6.7 1999/11/28 23:13:31 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "xstd.h"

#include <limits.h>
#include <unistd.h>

#include <iostream.h>
#include <iomanip.h>

#include "Assert.h"
#include "Clock.h"
#include "Poll.h"

#include "gadgets.h"


PollFD::PollFD(int aFd) {
	init(aFd);
}

void PollFD::init(int aFd) {
	this->fd = aFd;
	clear(dirNone);
}

void PollFD::set(IODir dir) {
	if (dir == dirRead)
		events |= XPOLL_RD;
	else
		events |= XPOLL_WR;
}

void PollFD::clear(IODir dir) {
	if (dir == dirRead) {
		events &= ~XPOLL_RD;
		revents &= ~XPOLL_RD;
	} else
	if (dir == dirWrite) {
		events &= ~XPOLL_WR;
		revents &= ~XPOLL_WR;
	} else
		events = 0;

	if (!events)
		revents = 0;
}


/* Poll */

Poll::Poll() {
}

void Poll::configure(int fdLimit) {
	FileScanner::configure(fdLimit);
	thePollees.stretch(fdLimit);
	thePollees.count(fdLimit);
	theHotFDs.stretch(fdLimit);

	for (int fd = 0; fd < thePollees.count(); ++fd)
		thePollees[fd].init(fd);
}

FileScanReserv Poll::setFD(int fd, IODir dir, FileScanUser *u) {
	Assert(fd < thePollees.count());
	thePollees[fd].set(dir);
	if (fd > theMaxFD)
		theMaxFD = fd;
	return FileScanner::setFD(fd, dir, u);
}

void Poll::clearFD(int fd, IODir dir) {
	thePollees[fd].clear(dir);

	if (fd == theMaxFD)
		while (--theMaxFD >= 0 && !thePollees[theMaxFD].used());

	FileScanner::clearFD(fd, dir);
}

FileScanUser *Poll::readyUser(int idx, IODir dir, int &fd) {
	const PollFD &pfd = theHotFDs[idx];
	fd = pfd.fd;

	// quit if poll(2) said "not ready"
	if (!pfd.ready(dir))
		return 0;

	// quit if fd is stale
	if (fd > theMaxFD)
		return 0;

	// quit if the user is no longer interested in this event
	// (most likely, it is a different user!)
	if (!thePollees[fd].interested(dir))
		return 0;

	return theRegs[fd].user();
}

int Poll::sweep(Time *timeout) {

	theHotFDs.reset();
	// copy active fds to hot array
	for (int fd = 0; fd <= theMaxFD; ++fd) {
		const PollFD &p = thePollees[fd];
		if (p.used() && theRegs[fd].needsCheck())
			theHotFDs.append(p);
	}

	if (!theHotFDs.count())
		return 0;

	const int res = xpoll(theHotFDs.items(), theHotFDs.count(),
		timeout ? timeout->msec() : -1);
	return res <= 0 ? res : theHotFDs.count();
}
