package ReportGen::Globs;
use strict;

# various command line options routines

require Exporter;
use ReportGen::ObjDbase;

@ReportGen::Globs::ISA = qw ( Exporter );
@ReportGen::Globs::EXPORT = qw ( 
	&GetCfg &FullInFName &FullOutFName 
	&ObjName2Type &Percent &System &MkDir );


my $RepDir = 'polyrep';
my $CfgFile = 'reportgen.cfg';

my %ObjTypes = (
	(map { $_ => 'rate (xact/sec)' }  qw ( rate )),
	(map { $_ => 'time (msec)' }  qw ( rptm.mean duration )),
	(map { $_ => 'count' } qw ( use.mean count level )),
	(map { $_ => 'ratio (%)' } qw ( ratio.obj ratio.byte ratio )),
	(map { $_ => 'size (bytes)' }  qw ( size.mean )),
	(map { $_ => 'string' } qw ( name )),
);

sub FullDir {
	my $label = shift;
	die() unless defined $label;
	my $dir = defined $ENV{TMPDIR} ? $ENV{TMPDIR} : '/tmp';
	return "$dir/$RepDir/$label";
}

sub FullInFName {
	my ($label, $fname, $ext) = @_;
	$ext ||= '';
	die("FullFName($label, $fname)") unless defined $fname;

	my $dir = &FullDir($label);
	return "$dir/$fname$ext";
}

sub FullOutFName {
	my ($label, $fname, $ext) = @_;
	$ext ||= '';
	die("FullFName($label, $fname)") unless defined $fname;

	my $dir = &FullDir($label);
	&MkDir($dir);
	return "$dir/$fname$ext";
}

sub ObjName2Type {
	my $name = shift;
	my ($n, $t);
	foreach my $n (keys %ObjTypes) {
		return $ObjTypes{$n} if $name =~ /$n/x;
	}
	return undef();
}

sub Percent {
	my ($n, $d) = @_;
	return -1 if abs($d) < 1e-6;
	return 100*$n/$d;
}

sub GetCfg {
	my $opts = ReadDbase($CfgFile);
	return $opts;
}

sub System {
	my ($cmd) = @_;

	die("$0: `$cmd' failed; probable cause: $!\n") if system($cmd);
}

sub MkDir {
	my ($dir) = @_;

	return if -d $dir;
	&System("mkdir -p $dir");
}
